# NAME

UV - A Weather Source API

Daily and hourly UV data for latitude/longitude points, ZIP/Postal Codes, and Designated Market Areas. UV forecast is up to 5 days out from the current day. Historical UV data is also available. The Weather Source UV API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.  When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page.

# VERSION

Automatically generated by the [Swagger Codegen](https://github.com/swagger-api/swagger-codegen) project:

- API version: 2.0.0
- Package version: 
- Build package: io.swagger.codegen.v3.generators.scala.ScalaClientCodegen
For more information, please visit [https://developer.weathersourceapis.com/](https://developer.weathersourceapis.com/)

# Requirements

Building the API client library requires [Maven](https://maven.apache.org/) to be installed.

## Installation

To install the API client library to your local Maven repository, simply execute:

```shell
mvn install
```

To deploy it to a remote Maven repository instead, configure the settings of the repository and execute:

```shell
mvn deploy
```

Refer to the [official documentation](https://maven.apache.org/plugins/maven-deploy-plugin/usage.html) for more information.

### Maven users

Add this dependency to your project's POM:

```xml
<dependency>
    <groupId>io.swagger</groupId>
    <artifactId>swagger-scala-client</artifactId>
    <version>1.0.0</version>
    <scope>compile</scope>
</dependency>
```

### Gradle users

Add this dependency to your project's build file:

```groovy
compile "io.swagger:swagger-scala-client:1.0.0"
```

### SBT users

```scala
libraryDependencies += "io.swagger" % "swagger-scala-client" % "1.0.0"
```

## Documentation for API Endpoints

All URIs are relative to *https://uv.weathersourceapis.com/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DmasApi* | [**dmasDmaIdDaysDateGet**](DmasApi.md#dmasDmaIdDaysDateGet) | **GET** /dmas/{dmaId}/days/{date} | A day of UV data for a DMA.
*DmasApi* | [**dmasDmaIdDaysDateStartdateEndGet**](DmasApi.md#dmasDmaIdDaysDateStartdateEndGet) | **GET** /dmas/{dmaId}/days/{dateStart},{dateEnd} | A contiguous range of daily UV data for a DMA.
*DmasApi* | [**dmasDmaIdDaysGet**](DmasApi.md#dmasDmaIdDaysGet) | **GET** /dmas/{dmaId}/days | The complete daily UV forecast for a DMA.
*DmasApi* | [**dmasDmaIdHoursGet**](DmasApi.md#dmasDmaIdHoursGet) | **GET** /dmas/{dmaId}/hours | The complete hourly UV forecast for a DMA
*DmasApi* | [**dmasDmaIdHoursTimestampGet**](DmasApi.md#dmasDmaIdHoursTimestampGet) | **GET** /dmas/{dmaId}/hours/{timestamp} | An hour of UV data for a DMA
*DmasApi* | [**dmasDmaIdHoursTimestampStarttimestampEndGet**](DmasApi.md#dmasDmaIdHoursTimestampStarttimestampEndGet) | **GET** /dmas/{dmaId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly UV data for a DMA.
*OnpointsApi* | [**onpointsOnpointIdDaysDateGet**](OnpointsApi.md#onpointsOnpointIdDaysDateGet) | **GET** /onpoints/{onpointId}/days/{date} | A day of UV data for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdDaysDateStartdateEndGet**](OnpointsApi.md#onpointsOnpointIdDaysDateStartdateEndGet) | **GET** /onpoints/{onpointId}/days/{dateStart},{dateEnd} | A contiguous range of daily UV data for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdDaysGet**](OnpointsApi.md#onpointsOnpointIdDaysGet) | **GET** /onpoints/{onpointId}/days | The complete daily UV forecast for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHoursGet**](OnpointsApi.md#onpointsOnpointIdHoursGet) | **GET** /onpoints/{onpointId}/hours | The complete hourly UV forecast for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHoursTimestampGet**](OnpointsApi.md#onpointsOnpointIdHoursTimestampGet) | **GET** /onpoints/{onpointId}/hours/{timestamp} | An hour of UV data for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHoursTimestampStarttimestampEndGet**](OnpointsApi.md#onpointsOnpointIdHoursTimestampStarttimestampEndGet) | **GET** /onpoints/{onpointId}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly UV data for an OnPoint™ point.
*PointsApi* | [**pointsLatitudelongitudeDaysDateGet**](PointsApi.md#pointsLatitudelongitudeDaysDateGet) | **GET** /points/{latitude},{longitude}/days/{date} | A day of UV data for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeDaysDateStartdateEndGet**](PointsApi.md#pointsLatitudelongitudeDaysDateStartdateEndGet) | **GET** /points/{latitude},{longitude}/days/{dateStart},{dateEnd} | A contiguous range of daily UV data for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeDaysGet**](PointsApi.md#pointsLatitudelongitudeDaysGet) | **GET** /points/{latitude},{longitude}/days | The complete daily UV forecast for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHoursGet**](PointsApi.md#pointsLatitudelongitudeHoursGet) | **GET** /points/{latitude},{longitude}/hours | The complete hourly UV forecast for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHoursTimestampGet**](PointsApi.md#pointsLatitudelongitudeHoursTimestampGet) | **GET** /points/{latitude},{longitude}/hours/{timestamp} | An hour of UV data for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHoursTimestampStarttimestampEndGet**](PointsApi.md#pointsLatitudelongitudeHoursTimestampStarttimestampEndGet) | **GET** /points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly UV data for a Latitude/Longitude point.
*PostcodesApi* | [**postcodesPostcodecountryCodeDaysDateGet**](PostcodesApi.md#postcodesPostcodecountryCodeDaysDateGet) | **GET** /postcodes/{postcode},{countryCode}/days/{date} | A day of UV data for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeDaysDateStartdateEndGet**](PostcodesApi.md#postcodesPostcodecountryCodeDaysDateStartdateEndGet) | **GET** /postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd} | A contiguous range of daily UV data for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeDaysGet**](PostcodesApi.md#postcodesPostcodecountryCodeDaysGet) | **GET** /postcodes/{postcode},{countryCode}/days | The complete daily UV forecast for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeHoursGet**](PostcodesApi.md#postcodesPostcodecountryCodeHoursGet) | **GET** /postcodes/{postcode},{countryCode}/hours | The complete hourly UV forecast for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeHoursTimestampGet**](PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestamp} | An hour of UV data for a postcode.
*PostcodesApi* | [**postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet**](PostcodesApi.md#postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet) | **GET** /postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd} | A contiguous range of hourly UV data for a postcode.

## Documentation for Models

 - [BoundingPoints](BoundingPoints.md)
 - [BoundingPointsInner](BoundingPointsInner.md)
 - [CountryCode](CountryCode.md)
 - [CountryName](CountryName.md)
 - [Date](Date.md)
 - [DateEnd](DateEnd.md)
 - [DateRangeObj](DateRangeObj.md)
 - [DateStart](DateStart.md)
 - [Distance](Distance.md)
 - [DistancePop](DistancePop.md)
 - [DmaId](DmaId.md)
 - [DmaName](DmaName.md)
 - [DmaObj](DmaObj.md)
 - [Elevation](Elevation.md)
 - [ErrorObj](ErrorObj.md)
 - [FieldListDayObj](FieldListDayObj.md)
 - [FieldListHourObj](FieldListHourObj.md)
 - [Fields](Fields.md)
 - [Grid](Grid.md)
 - [Latitude](Latitude.md)
 - [Longitude](Longitude.md)
 - [OnpointId](OnpointId.md)
 - [OnpointObj](OnpointObj.md)
 - [PointObj](PointObj.md)
 - [Population](Population.md)
 - [Postcode](Postcode.md)
 - [PostcodeObj](PostcodeObj.md)
 - [SamplePoints](SamplePoints.md)
 - [SamplePointsInner](SamplePointsInner.md)
 - [SubdivCode](SubdivCode.md)
 - [SubdivName](SubdivName.md)
 - [Timestamp](Timestamp.md)
 - [TimestampEnd](TimestampEnd.md)
 - [TimestampInit](TimestampInit.md)
 - [TimestampRangeObj](TimestampRangeObj.md)
 - [TimestampStart](TimestampStart.md)
 - [Timezone](Timezone.md)
 - [UvDayArr](UvDayArr.md)
 - [UvDayObj](UvDayObj.md)
 - [UvDmaDayObj](UvDmaDayObj.md)
 - [UvDmaHourObj](UvDmaHourObj.md)
 - [UvHourArr](UvHourArr.md)
 - [UvHourObj](UvHourObj.md)
 - [UvIndex](UvIndex.md)
 - [UvIndexMax](UvIndexMax.md)
 - [UvOnpointDayObj](UvOnpointDayObj.md)
 - [UvOnpointHourObj](UvOnpointHourObj.md)
 - [UvPointDayObj](UvPointDayObj.md)
 - [UvPointHourObj](UvPointHourObj.md)
 - [UvPostcodeDayObj](UvPostcodeDayObj.md)
 - [UvPostcodeHourObj](UvPostcodeHourObj.md)

## Documentation for Authorization

Authentication schemes defined for the API:
### ApiKeyAuth

- **Type**: API key
- **API key parameter name**: X-API-KEY
- **Location**: HTTP header



# BUILDING YOUR LIBRARY

See the homepage `https://github.com/swagger-api/swagger-codegen` for full details.
But briefly, clone the git repository, build the codegen codebase, set up your build
config file, then run the API build script. You will need git, Java 7 or 8 and Apache
maven 3.0.3 or better already installed.

Your library files will be built under `WWW::MyProjectName`.

          $ git clone https://github.com/swagger-api/swagger-codegen.git
          $ cd swagger-codegen
          $ mvn package
          $ java -jar modules/swagger-codegen-cli/target/swagger-codegen-cli.jar generate \
    -i [URL or file path to JSON swagger API spec] \
    -l akka-scala \
    -c /path/to/config/file.json \
    -o /path/to/output/folder

Bang, all done. Run the `autodoc` script in the `bin` directory to see the API
you just built.

## Author

support@weathersource.com
