/**
 * UV - A Weather Source API
 * Daily and hourly UV data for latitude/longitude points, ZIP/Postal Codes, and Designated Market Areas. UV forecast is up to 5 days out from the current day. Historical UV data is also available. The Weather Source UV API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.  When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page.
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */
package io.swagger.client.api

import java.text.SimpleDateFormat

import java.util.Date
import io.swagger.client.model.date
import io.swagger.client.model.errorObj
import io.swagger.client.model.uvPostcodeDayObj
import io.swagger.client.model.uvPostcodeHourObj
import io.swagger.client.{ApiInvoker, ApiException}

import com.sun.jersey.multipart.FormDataMultiPart
import com.sun.jersey.multipart.file.FileDataBodyPart

import javax.ws.rs.core.MediaType

import java.io.File
import java.util.Date
import java.util.TimeZone

import scala.collection.mutable.HashMap

import com.wordnik.swagger.client._
import scala.concurrent.Future
import collection.mutable

import java.net.URI

import com.wordnik.swagger.client.ClientResponseReaders.Json4sFormatsReader._
import com.wordnik.swagger.client.RequestWriters.Json4sFormatsWriter._

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent._
import scala.concurrent.duration._
import scala.util.{Failure, Success, Try}

import org.json4s._

class PostcodesApi(
  val defBasePath: String = "https://uv.weathersourceapis.com/v2",
  defApiInvoker: ApiInvoker = ApiInvoker
) {
  private lazy val dateTimeFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  private val dateFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  implicit val formats = new org.json4s.DefaultFormats {
    override def dateFormatter = dateTimeFormatter
  }
  implicit val stringReader: ClientResponseReader[String] = ClientResponseReaders.StringReader
  implicit val unitReader: ClientResponseReader[Unit] = ClientResponseReaders.UnitReader
  implicit val jvalueReader: ClientResponseReader[JValue] = ClientResponseReaders.JValueReader
  implicit val jsonReader: ClientResponseReader[Nothing] = JsonFormatsReader
  implicit val stringWriter: RequestWriter[String] = RequestWriters.StringWriter
  implicit val jsonWriter: RequestWriter[Nothing] = JsonFormatsWriter

  var basePath: String = defBasePath
  var apiInvoker: ApiInvoker = defApiInvoker

  def addHeader(key: String, value: String): mutable.HashMap[String, String] = {
    apiInvoker.defaultHeaders += key -> value
  }

  val config: SwaggerConfig = SwaggerConfig.forUrl(new URI(defBasePath))
  val client = new RestClient(config)
  val helper = new PostcodesApiAsyncHelper(client, config)

  /**
   * A day of UV data for a postcode.
   * Returns a single day of UV data for a postcode. The provided date must exist within the current valid 5-day forecast range, or it must be a historical date.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param date An date formatted as a RFC3339 date value. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeDayObj
   */
  def postcodesPostcodecountryCodeDaysDateGet(postcode: String, countryCode: String, date: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeDayObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeDaysDateGetAsync(postcode, countryCode, date, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A day of UV data for a postcode. asynchronously
   * Returns a single day of UV data for a postcode. The provided date must exist within the current valid 5-day forecast range, or it must be a historical date.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param date An date formatted as a RFC3339 date value. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeDayObj)
   */
  def postcodesPostcodecountryCodeDaysDateGetAsync(postcode: String, countryCode: String, date: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeDayObj] = {
      helper.postcodesPostcodecountryCodeDaysDateGet(postcode, countryCode, date, fields, unitScale)
  }

  /**
   * A contiguous range of daily UV data for a postcode.
   * Returns a contiguous range of daily ECMWF forecast data for a postcode. The temporal range provided must not exceed 1 year in duration.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
   * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeDayObj
   */
  def postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: String, countryCode: String, dateStart: date, dateEnd: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeDayObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeDaysDateStartdateEndGetAsync(postcode, countryCode, dateStart, dateEnd, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A contiguous range of daily UV data for a postcode. asynchronously
   * Returns a contiguous range of daily ECMWF forecast data for a postcode. The temporal range provided must not exceed 1 year in duration.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
   * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeDayObj)
   */
  def postcodesPostcodecountryCodeDaysDateStartdateEndGetAsync(postcode: String, countryCode: String, dateStart: date, dateEnd: date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeDayObj] = {
      helper.postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode, countryCode, dateStart, dateEnd, fields, unitScale)
  }

  /**
   * The complete daily UV forecast for a postcode.
   * Returns the complete set of UV forecast data for a postcode.  *Alternatively using a query that specificies just the days required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeDayObj
   */
  def postcodesPostcodecountryCodeDaysGet(postcode: String, countryCode: String, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeDayObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeDaysGetAsync(postcode, countryCode, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * The complete daily UV forecast for a postcode. asynchronously
   * Returns the complete set of UV forecast data for a postcode.  *Alternatively using a query that specificies just the days required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeDayObj)
   */
  def postcodesPostcodecountryCodeDaysGetAsync(postcode: String, countryCode: String, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeDayObj] = {
      helper.postcodesPostcodecountryCodeDaysGet(postcode, countryCode, fields, unitScale)
  }

  /**
   * The complete hourly UV forecast for a postcode.
   * Returns the complete set of UV forecast data for a postcode.  *Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeHourObj
   */
  def postcodesPostcodecountryCodeHoursGet(postcode: String, countryCode: String, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeHourObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeHoursGetAsync(postcode, countryCode, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * The complete hourly UV forecast for a postcode. asynchronously
   * Returns the complete set of UV forecast data for a postcode.  *Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeHourObj)
   */
  def postcodesPostcodecountryCodeHoursGetAsync(postcode: String, countryCode: String, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeHourObj] = {
      helper.postcodesPostcodecountryCodeHoursGet(postcode, countryCode, fields, unitScale)
  }

  /**
   * An hour of UV data for a postcode.
   * Returns a single hour of ECMWF forecast data for a postcode. The provided timestamp must exist within the current valid 5-day forecast range, or it must be a historical timestamp.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeHourObj
   */
  def postcodesPostcodecountryCodeHoursTimestampGet(postcode: String, countryCode: String, timestamp: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeHourObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeHoursTimestampGetAsync(postcode, countryCode, timestamp, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * An hour of UV data for a postcode. asynchronously
   * Returns a single hour of ECMWF forecast data for a postcode. The provided timestamp must exist within the current valid 5-day forecast range, or it must be a historical timestamp.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeHourObj)
   */
  def postcodesPostcodecountryCodeHoursTimestampGetAsync(postcode: String, countryCode: String, timestamp: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeHourObj] = {
      helper.postcodesPostcodecountryCodeHoursTimestampGet(postcode, countryCode, timestamp, fields, unitScale)
  }

  /**
   * A contiguous range of hourly UV data for a postcode.
   * Returns a contiguous range of hourly UV data for a postcode. The temporal range provided must not exceed 1 year in duration.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return uvPostcodeHourObj
   */
  def postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: String, countryCode: String, timestampStart: Date, timestampEnd: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Option[uvPostcodeHourObj] = {
    val await = Try(Await.result(postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetAsync(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A contiguous range of hourly UV data for a postcode. asynchronously
   * Returns a contiguous range of hourly UV data for a postcode. The temporal range provided must not exceed 1 year in duration.  **IMPORTANT:** To authenticate to the Weather Source UV API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param postcode A postcode string. 
   * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/). 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)*  
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(uvPostcodeHourObj)
   */
  def postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetAsync(postcode: String, countryCode: String, timestampStart: Date, timestampEnd: Date, fields: List[String], unitScale: Option[String] = Option("IMPERIAL")): Future[uvPostcodeHourObj] = {
      helper.postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode, countryCode, timestampStart, timestampEnd, fields, unitScale)
  }

}

class PostcodesApiAsyncHelper(client: TransportClient, config: SwaggerConfig) extends ApiClient(client, config) {

  def postcodesPostcodecountryCodeDaysDateGet(postcode: String,
    countryCode: String,
    date: date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeDayObj]): Future[uvPostcodeDayObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/days/{date}")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString)
      replaceAll("\\{" + "date" + "\\}", date.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def postcodesPostcodecountryCodeDaysDateStartdateEndGet(postcode: String,
    countryCode: String,
    dateStart: date,
    dateEnd: date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeDayObj]): Future[uvPostcodeDayObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd}")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString)
      replaceAll("\\{" + "dateStart" + "\\}", dateStart.toString)
      replaceAll("\\{" + "dateEnd" + "\\}", dateEnd.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateStartdateEndGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateStartdateEndGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeDaysDateStartdateEndGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def postcodesPostcodecountryCodeDaysGet(postcode: String,
    countryCode: String,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeDayObj]): Future[uvPostcodeDayObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/days")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeDaysGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeDaysGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def postcodesPostcodecountryCodeHoursGet(postcode: String,
    countryCode: String,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeHourObj]): Future[uvPostcodeHourObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/hours")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeHoursGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def postcodesPostcodecountryCodeHoursTimestampGet(postcode: String,
    countryCode: String,
    timestamp: Date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeHourObj]): Future[uvPostcodeHourObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/hours/{timestamp}")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString)
      replaceAll("\\{" + "timestamp" + "\\}", timestamp.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(postcode: String,
    countryCode: String,
    timestampStart: Date,
    timestampEnd: Date,
    fields: List[String],
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[uvPostcodeHourObj]): Future[uvPostcodeHourObj] = {
    // create path and map variables
    val path = (addFmt("/postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd}")
      replaceAll("\\{" + "postcode" + "\\}", postcode.toString)
      replaceAll("\\{" + "countryCode" + "\\}", countryCode.toString)
      replaceAll("\\{" + "timestampStart" + "\\}", timestampStart.toString)
      replaceAll("\\{" + "timestampEnd" + "\\}", timestampEnd.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (postcode == null) throw new Exception("Missing required parameter 'postcode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet")

    if (countryCode == null) throw new Exception("Missing required parameter 'countryCode' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet")

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling PostcodesApi->postcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }


}
